%clear all
%clc
%colordef none;

% General parameters
fclk=50e6;
T=1/fclk;
f = logspace(3,9,1e4);

% TDC parameters
deltat=6e-12;
deltaf=1e-12;
fgain=1.2e-20

% DAC parameters
V=1.5;
B=10;
cu=30e-15;
c1=cu*32;
c2_f=2.5e-12;
c2_c=20e-12;
req_f=1/fclk/c1;
req_c=1/fclk/c1;
fp_f=1/2/pi/req_f/c2_f;
fp_c=1/2/pi/req_c/c2_c;
Hlp_f=1./(1+j*f./fp_f);
theta=(4e-9)^2/4/1e3;
svn_f=4*theta*req_f./(1+(f/fp_f).^2);
svn_c=4*theta*req_c./(1+(f/fp_c).^2);

% VCO parameters
Kvf=5e6; 
Kvc=80e6;
Phin_vco = -150; %dBc/Hz
foffset = 20e6;
fcorner=200e3; %1/f corner
Vnvco_f = (10^(Phin_vco/10))*(foffset/Kvf)*(foffset/Kvf).*(1+fcorner./f); %V^2/Hz spectral density - SSB Noise
Vnvco_s = 2*pi*Vnvco_f; %V^2/rad spectral density
H_vco_f = (2*pi*Kvf)./(2*pi*j*f); % fine varactor
S_vco_f = H_vco_f.*conj(H_vco_f); 
phin_vco_f = Vnvco_f.*S_vco_f;

% filter parameters
K1=127.3; % from par_calc.m
K2=0.0513; % from par_calc.m
alpha=0.8961; % from par_calc.m
H=K2*(1-alpha)./(1-alpha.*exp(-j*2*pi*f*T)).*(K1+1./(1-exp(-j*2*pi*f*T)));

% divider parameters
Nnom=73;
m=3;
left=0.01^2;

% reference parameters
Kv_ref = 1; %Hz/V!
Phin_ref = -150; %dBc/Hz 
foffset_ref = 1e3; 
fcorner_ref=0e3; %1/f corner
floor=-300;%dBc/Hz 
Vnref_f = (10^(Phin_ref/10))*(foffset_ref/Kv_ref)*(foffset_ref/Kv_ref).*(1+fcorner_ref./f); %V^2/Hz spectral density - SSB Noise
Vnref_s = 2*pi*Vnref_f; %V^2/rad spectral density
H_ref = (2*pi*Kv_ref)./(2*pi*j*f);
S_ref = H_ref.*conj(H_ref); 
phin_ref_f = 1* (Vnref_f.*S_ref+10^(floor/10));

a=T/(2*pi)*(1/deltat).*H*(V/2^B).*Hlp_f.*(Kvf./(j*f))/Nnom;
g=a./(1+a);

% TDC noise
So_traw=1/T.*(abs(2*pi*Nnom*g)).^2.*((2*sin(pi*f*T)).^2*(deltat^2/12)+(deltaf)^2/12+fgain./f);
%figure(2);
%semilogx(f,10*log10(So_traw));

% Divider noise
So_r=left*T.*(abs(2*pi*g)).^2.*(2*sin(pi*f*T)).^(2*(m-1))*(1/12);
%figure(4);
%semilogx(f,10*log10(So_r));

% VCO noise
So_phin=(abs(1-g)).^2.*phin_vco_f;
%figure(5);
%semilogx(f,10*log10(So_phin));

% Fine DAC quantization noise
So_tr=T*(abs(1-g)).^2*(V/2^B)^2.*(abs(Hlp_f)).^2.*(Kvf./f).^2.*(2*sin(pi*f*T)).^2*(1/12);
%figure(6);
%semilogx(f,10*log10(So_tr));

% Fine DAC thermal noise
So_vnf=(abs(1-g)).^2.*(Kvf./f).^2.*svn_f;

% Coarse DAC thermal noise
So_vnc=(abs(1-g)).^2.*(Kvc./f).^2.*svn_c;

% reference noise
So_ref=(abs(Nnom*g)).^2.*phin_ref_f;

% Total Noise
So=So_traw+So_r+So_phin+So_tr+So_vnf+So_vnc+So_ref;
%figure(7);
%semilogx(f,10*log10(So));

%GSM Mask
mask_values = [-52.3 -82.8 -85.8 -112.8 -112.8 -121 -123 -129 -150 -162]+12;
f_mask = [100e3 200e3 250e3 400e3 600e3 1.8e6 3.0e6 6.0e6 10e6 20e6];

%hold on;
%semilogx(f,10*log10(So_phin),'r',f,10*log10(So_tr),'b',f,10*log10(So_vnf),'k--',f,10*log10(So_vnc),'k',f,10*log10(So_r),'c',f,10*log10(So_traw),'g',f,10*log10(So_ref),'g--',f,10*log10(So),'m',f_mask,mask_values,'or-');
semilogx(f,10*log10(So_phin),'r',f,10*log10(So_tr),'b',f,10*log10(So_vnf),'k--',f,10*log10(So_vnc),'k',f,10*log10(So_r),'c',f,10*log10(So_traw),'g',f,10*log10(So_ref),'g--',f,10*log10(So),'m');
grid on
axis([1e3 40e6 -180 -40])
ylabel('dBc/Hz')
xlabel('f_{offset}')
%legend('Measured Noise','VCO Noise','Finepath \Sigma\Delta Quantization Noise','Fine-tune DAC Thermal','Coarse-tune DAC Thermal','Divider Noise (100% left)','GRO Noise','Ref Noise','Close-loop Noise')
legend('VCO Noise','Finepath \Sigma\Delta Quantization Noise','Fine-tune DAC Thermal','Coarse-tune DAC Thermal','Divider Noise (1% left)','GRO Noise','Ref Noise','Close-loop Noise')


% open-loop VCO noise
% Sdco=phin_vco_f+(Kvf./f).^2.*svn_f+(Kvc./f).^2.*svn_c;
% figure(2);
% semilogx(f,10*log10(phin_vco_f),f,10*log10(Sdco));
